@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2024, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -20,15 +20,15 @@ init
 json
     Create JSON files for one or more applications
 
-sampledata
-    Download the bokeh sample data sets
-
 secret
     Create a Bokeh secret key for use with Bokeh server
 
 serve
     Run a Bokeh server hosting one or more applications
 
+settings
+    Print information about Bokeh settings and their current values
+
 static
     Serve bokehjs' static assets (JavaScript, CSS, images, fonts, etc.)
 
