@@ -68,6 +68,7 @@ from .callbacks import (
     JSEventCallback,
     MessageCallback,
 )
+from .config import DocumentConfig
 from .events import (
     DocumentPatchedEvent,
     RootAddedEvent,
@@ -124,6 +125,7 @@ class Document:
     models: DocumentModelManager
     modules: DocumentModuleManager
 
+    _config: DocumentConfig
     _roots: list[Model]
     _theme: Theme
     _title: str
@@ -136,6 +138,7 @@ class Document:
         self.models = DocumentModelManager(self)
         self.modules = DocumentModuleManager(self)
 
+        self._config = DocumentConfig()
         self._roots = []
         self._template = FILE
         self._template_variables = {}
@@ -146,9 +149,16 @@ class Document:
 
     # Properties --------------------------------------------------------------
 
+    @property
+    def config(self) -> DocumentConfig:
+        ''' A configuration of this document.
+
+        '''
+        return self._config
+
     @property
     def roots(self) -> list[Model]:
-        ''' A list of all the root models in this Document.
+        ''' A list of all the root models in this document.
 
         '''
         return list(self._roots)
@@ -350,7 +360,7 @@ class Document:
         ''' Apply a JSON patch object and process any resulting events.
 
         Args:
-            patch (JSON-data) :
+            patch_json (JSON-data) :
                 The JSON-object containing the patch to apply.
 
             setter (ClientSession or ServerSession or None, optional) :
@@ -437,10 +447,13 @@ side of a communications channel while it was being removed on the other end.\
         deserializer = Deserializer()
         doc_struct = deserializer.deserialize(doc_json)
 
+        config = doc_struct["config"]
         roots = doc_struct["roots"]
         title = doc_struct["title"]
 
         doc = Document()
+        doc._config = config
+
         for root in roots:
             doc.add_root(root)
 
@@ -496,7 +509,7 @@ side of a communications channel while it was being removed on the other end.\
         hold will be applied according to the hold policy.
 
         Args:
-            hold ('combine' or 'collect', optional)
+            policy ('combine' or 'collect', optional)
                 Whether events collected during a hold should attempt to be
                 combined (default: 'combine')
 
@@ -716,7 +729,7 @@ side of a communications channel while it was being removed on the other end.\
         Args:
             selector (JSON-like query dictionary) : you can query by type or by
                 name,i e.g. ``{"type": HoverTool}``, ``{"name": "mycircle"}``
-                updates (dict) :
+            updates (dict) :
 
         Returns:
             None
@@ -739,7 +752,7 @@ side of a communications channel while it was being removed on the other end.\
             self.callbacks.trigger_on_change(TitleChangedEvent(self, title, setter))
 
     def to_json(self, *, deferred: bool = True) -> DocJson:
-        ''' Convert this document to a JSON-serializble object.
+        ''' Convert this document to a JSON-serializable object.
 
         Return:
             DocJson
@@ -749,12 +762,14 @@ side of a communications channel while it was being removed on the other end.\
 
         serializer = Serializer(deferred=deferred)
         defs = serializer.encode(data_models)
+        config = serializer.encode(self._config)
         roots = serializer.encode(self._roots)
         callbacks = serializer.encode(self.callbacks._js_event_callbacks)
 
         doc_json = DocJson(
             version=__version__,
             title=self.title,
+            config=config,
             roots=roots,
         )
 
@@ -809,6 +824,7 @@ side of a communications channel while it was being removed on the other end.\
         # we have to remove ALL roots before adding any
         # to the new doc or else models referenced from multiple
         # roots could be in both docs at once, which isn't allowed.
+        config = self.config
         roots: list[Model] = []
 
         with self.models.freeze():
@@ -824,6 +840,8 @@ side of a communications channel while it was being removed on the other end.\
         if len(self.models) != 0:
             raise RuntimeError(f"_all_models still had stuff in it: {self.models!r}")
 
+        dest_doc._config = config
+
         for root in roots:
             dest_doc.add_root(root)
 
