@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2024, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -20,6 +20,9 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any
+
 # Bokeh imports
 from ..colors import RGB, Color, ColorLike
 from ..core.enums import (
@@ -64,7 +67,8 @@ from ..core.validation.warnings import (
     FIXED_WIDTH_POLICY,
 )
 from ..model import Model
-from .ui.menus import Menu
+from .ui.panes import Pane
+from .ui.tooltips import Tooltip
 from .ui.ui_element import UIElement
 
 #-----------------------------------------------------------------------------
@@ -91,13 +95,13 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 @abstract
-class LayoutDOM(UIElement):
+class LayoutDOM(Pane):
     """ The base class for layoutable components.
 
     """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     disabled = Bool(False, help="""
@@ -106,13 +110,13 @@ class LayoutDOM(UIElement):
     If ``True``, the widget will be greyed-out and not responsive to UI events.
     """)
 
-    width: int | None = Nullable(NonNegative(Int), help="""
+    width = Nullable(NonNegative(Int), help="""
     The width of the component (in pixels).
 
     This can be either fixed or preferred width, depending on width sizing policy.
     """)
 
-    height: int | None = Nullable(NonNegative(Int), help="""
+    height = Nullable(NonNegative(Int), help="""
     The height of the component (in pixels).
 
     This can be either fixed or preferred height, depending on height sizing policy.
@@ -274,15 +278,6 @@ class LayoutDOM(UIElement):
     grid track align).
     """)
 
-    context_menu = Nullable(Instance(Menu), default=None, help="""
-    A menu to display when user right clicks on the component.
-
-    .. note::
-        To display a native context menu, the viewer will need to right-click
-        twice. The second click closes the Bokeh context menu and falls back
-        back the native one.
-    """)
-
     resizable = Either(Bool, Enum(Dimensions), default=False, help="""
     Whether the layout is interactively resizable, and if so in which dimensions.
     """)
@@ -317,18 +312,18 @@ class LayoutDOM(UIElement):
             return str(self)
 
     @error(MIN_PREFERRED_MAX_WIDTH)
-    def _min_preferred_max_width(self):
+    def _check_min_preferred_max_width(self):
         min_width = self.min_width if self.min_width is not None else 0
-        width     = self.width     if self.width     is not None else min_width
+        width     = self.width     if self.width     is not None and (self.sizing_mode == "fixed" or self.width_policy == "fixed") else min_width
         max_width = self.max_width if self.max_width is not None else width
 
         if not (min_width <= width <= max_width):
             return str(self)
 
     @error(MIN_PREFERRED_MAX_HEIGHT)
-    def _min_preferred_max_height(self):
+    def _check_min_preferred_max_height(self):
         min_height = self.min_height if self.min_height is not None else 0
-        height     = self.height     if self.height     is not None else min_height
+        height     = self.height     if self.height     is not None and (self.sizing_mode == "fixed" or self.height_policy == "fixed") else min_height
         max_height = self.max_height if self.max_height is not None else height
 
         if not (min_height <= height <= max_height):
@@ -345,7 +340,7 @@ class Spacer(LayoutDOM):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -402,7 +397,7 @@ class GridBox(LayoutDOM, GridCommon):
     """ A CSS grid-based grid container. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     children = List(Either(
@@ -421,7 +416,7 @@ class HBox(LayoutDOM):
     """ A CSS grid-based horizontal box. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     children = List(Struct(child=Instance(UIElement), col=Optional(Int), span=Optional(Int)), default=[], help="""
@@ -438,7 +433,7 @@ class HBox(LayoutDOM):
 
     @error(REPEATED_LAYOUT_CHILD)
     def _check_repeated_layout_children(self):
-        children = [ item["child"] for item in self.items ]
+        children = [ item["child"] for item in self.children ]
         if len(children) != len(set(children)):
             return str(self)
 
@@ -446,7 +441,7 @@ class VBox(LayoutDOM):
     """ A CSS grid-based vertical box. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     children = List(Struct(child=Instance(UIElement), row=Optional(Int), span=Optional(Int)), default=[], help="""
@@ -463,7 +458,7 @@ class VBox(LayoutDOM):
 
     @error(REPEATED_LAYOUT_CHILD)
     def _check_repeated_layout_children(self):
-        children = [ item["child"] for item in self.items ]
+        children = [ item["child"] for item in self.children ]
         if len(children) != len(set(children)):
             return str(self)
 
@@ -521,7 +516,7 @@ class Row(FlexBox):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     def _sphinx_height_hint(self) -> int|None:
@@ -537,7 +532,7 @@ class Column(FlexBox):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     def _sphinx_height_hint(self) -> int|None:
@@ -551,13 +546,18 @@ class TabPanel(Model):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     title = String(default="", help="""
     The text title of the panel.
     """)
 
+    tooltip = Nullable(Instance(Tooltip), default=None, help="""
+    A tooltip with plain text or rich HTML contents, providing general help or
+    description of a widget's or component's function.
+    """)
+
     child = Instance(UIElement, help="""
     The child widget. If you need more children, use a layout widget, e.g. a ``Column``.
     """)
@@ -578,7 +578,7 @@ class Tabs(LayoutDOM):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     __example__ = "examples/interaction/widgets/tab_panes.py"
@@ -602,7 +602,7 @@ class GroupBox(LayoutDOM):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     title = Nullable(String, help="""
@@ -626,7 +626,7 @@ class ScrollBox(LayoutDOM):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     child = Instance(UIElement, help="""
