@@ -21,11 +21,16 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any
+
 # Bokeh imports
 from ..core.enums import LatLon
 from ..core.has_props import abstract
 from ..core.properties import (
+    AnyRef,
     Auto,
+    Dict,
     Either,
     Enum,
     Float,
@@ -36,6 +41,7 @@ from ..core.properties import (
     Override,
     Required,
     Seq,
+    String,
 )
 from ..core.validation import error
 from ..core.validation.errors import MISSING_MERCATOR_DIMENSION
@@ -47,21 +53,23 @@ from .mappers import ScanningColorMapper
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'Ticker',
-    'BinnedTicker',
-    'ContinuousTicker',
-    'FixedTicker',
     'AdaptiveTicker',
-    'CompositeTicker',
-    'SingleIntervalTicker',
-    'DaysTicker',
-    'MonthsTicker',
-    'YearsTicker',
     'BasicTicker',
+    'BinnedTicker',
+    'CategoricalTicker',
+    'CompositeTicker',
+    'ContinuousTicker',
+    'CustomJSTicker',
+    'DatetimeTicker',
+    'DaysTicker',
+    'FixedTicker',
     'LogTicker',
     'MercatorTicker',
-    'CategoricalTicker',
-    'DatetimeTicker',
+    'MonthsTicker',
+    'SingleIntervalTicker',
+    'Ticker',
+    'TimedeltaTicker',
+    'YearsTicker',
 )
 
 #-----------------------------------------------------------------------------
@@ -75,9 +83,81 @@ class Ticker(Model):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
+class CustomJSTicker(Ticker):
+    ''' Generate tick locations that are computed by a user-defined function.
+
+    A ``CustomJSTicker`` may be used with either a continuous (numeric) axis,
+    or a categorical axis. However, only basic, non-hierarchical categorical
+    axes (i.e. with a single level of factors) are supported.
+
+    .. warning::
+        The explicit purpose of this Bokeh Model is to embed *raw JavaScript
+        code* for a browser to execute. If any part of the code is derived
+        from untrusted user inputs, then you must take appropriate care to
+        sanitize the user input prior to passing to Bokeh.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    args = Dict(String, AnyRef, help="""
+    A mapping of names to Python objects. In particular those can be bokeh's models.
+    These objects are made available to the ticker's code snippet as the values of
+    named parameters to the callback.
+    """)
+
+    major_code = String(default="", help="""
+    Callback code to run in the browser to compute minor tick locations for the
+    current viewport.
+
+    The ``cb_data`` parameter that is available to the callback code will contain
+    four specific fields:
+
+    ``start``
+        the computed start coordinate of the axis
+
+    ``end``
+        the computed end of the axis
+
+    ``range``
+        the Range model for this axis
+
+    ``cross_loc``
+        the coordinate that this axis intersects the orthogonal axis
+    """)
+
+    minor_code = String(default="", help="""
+    Callback code to run in the browser to compute minor tick locations for the
+    current viewport.
+
+    .. note::
+        Minor ticks are not used for categorical axes. This property will be
+        ignored when the range is a ``FactorRange``.
+
+    The ``cb_data`` parameter that is available to the callback code will contain
+    five specific fields:
+
+    ``major_ticks``
+        the list of the current computed major tick locations
+
+    ``start``
+        the computed start coordinate of the axis
+
+    ``end``
+        the computed end of the axis
+
+    ``range``
+        the Range model for this axis
+
+    ``cross_loc``
+        the coordinate that this axis intersects the orthogonal axis
+    """)
+
 @abstract
 class ContinuousTicker(Ticker):
     ''' A base class for non-categorical ticker types.
@@ -85,7 +165,7 @@ class ContinuousTicker(Ticker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     num_minor_ticks = Int(5, help="""
@@ -112,7 +192,7 @@ class FixedTicker(ContinuousTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     ticks = Seq(Float, default=[], help="""
@@ -135,7 +215,7 @@ class AdaptiveTicker(ContinuousTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     base = Float(10.0, help="""
@@ -167,7 +247,7 @@ class CompositeTicker(ContinuousTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     tickers = NonEmpty(Seq(Instance(Ticker)), help="""
@@ -185,7 +265,7 @@ class BaseSingleIntervalTicker(ContinuousTicker):
     ''' Base class for single interval tickers. '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class SingleIntervalTicker(BaseSingleIntervalTicker):
@@ -195,7 +275,7 @@ class SingleIntervalTicker(BaseSingleIntervalTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     interval = Required(Float, help="""
@@ -208,7 +288,7 @@ class DaysTicker(BaseSingleIntervalTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     days = Seq(Int, default=[], help="""
@@ -223,7 +303,7 @@ class MonthsTicker(BaseSingleIntervalTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     months = Seq(Int, default=[], help="""
@@ -236,7 +316,7 @@ class YearsTicker(BaseSingleIntervalTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class BasicTicker(AdaptiveTicker):
@@ -248,7 +328,7 @@ class BasicTicker(AdaptiveTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class LogTicker(AdaptiveTicker):
@@ -257,7 +337,7 @@ class LogTicker(AdaptiveTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     mantissas = Override(default=[1, 5])
@@ -269,7 +349,7 @@ class MercatorTicker(BasicTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     dimension = Nullable(Enum(LatLon), help="""
@@ -300,7 +380,7 @@ class CategoricalTicker(Ticker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 ONE_MILLI = 1.0
@@ -317,7 +397,7 @@ class DatetimeTicker(CompositeTicker):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     num_minor_ticks = Override(default=0)
@@ -358,13 +438,56 @@ class DatetimeTicker(CompositeTicker):
         YearsTicker(),
     ])
 
+class TimedeltaTicker(CompositeTicker):
+    ''' Generate nice ticks across different date and time scales.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    num_minor_ticks = Override(default=0)
+
+    # TODO: (bev) InstanceDefault for this, someday
+    tickers = Override(default=lambda: [
+        AdaptiveTicker(
+            mantissas=[1, 2, 5],
+            base=10,
+            min_interval=0,
+            max_interval=500*ONE_MILLI,
+            num_minor_ticks=0,
+        ),
+        AdaptiveTicker(
+            mantissas=[1, 2, 5, 10, 15, 20, 30],
+            base=60,
+            min_interval=ONE_SECOND,
+            max_interval=30*ONE_MINUTE,
+            num_minor_ticks=0,
+        ),
+        AdaptiveTicker(
+            mantissas=[1, 2, 4, 6, 8, 12],
+            base=24,
+            min_interval=ONE_HOUR,
+            max_interval=12*ONE_HOUR,
+            num_minor_ticks=0,
+        ),
+        AdaptiveTicker(
+            mantissas=[1, 2, 5],
+            base=10,
+            min_interval=ONE_DAY,
+            max_interval=None,
+            num_minor_ticks=0,
+        ),
+    ])
+
 class BinnedTicker(Ticker):
     """ Ticker that aligns ticks exactly at bin boundaries of a scanning color mapper.
 
     """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     mapper = Instance(ScanningColorMapper, help="""
